/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/

module data_ctrl(
    clk    ,
    rst_n  ,
    key_vld,
    wr_ack ,
    rd_ack ,
    wr_req ,
    rd_req ,
    bank   ,
    addr   ,
    wdata  
);


input          clk      ;
input          rst_n    ;
input  [3:0]   key_vld  ;
input          wr_ack   ;
input          rd_ack   ;
output         wr_req   ;
output         rd_req   ;
output [1:0]   bank     ;
output [12:0]  addr     ;
output [15:0]  wdata    ;

reg            wr_req   ;
reg            rd_req   ;
reg    [15:0]  wdata    ;
reg            flag_wr  ;
wire   [12:0]  addr     ;
reg    [9:0]   cnt2     ;
wire           add_cnt2 ;
wire           end_cnt2 ;
reg    [1:0]   bank     ;
wire           add_bank ;
wire           end_bank ;



always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        wr_req <= 0;
    end
    else if(key_vld[0]==1)begin
        wr_req <= 1;
    end
    else if(wr_ack)begin
        wr_req <= 0;
    end
end


always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        rd_req <= 0;
    end
    else if(key_vld[1]==1)begin
        rd_req <= 1;
    end
    else if(rd_ack)begin
        rd_req <= 0;
    end
end


always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        bank <= 0; 
    end
    else if(add_bank) begin
        if(end_bank)
            bank <= 0; 
        else
            bank <= bank+1 ;
   end
end
assign add_bank = key_vld[2]==1;
assign end_bank = add_bank  && bank == 4-1 ;


always  @(*)begin
    if(flag_wr)begin
        wdata = {7'b0,cnt2};
    end
    else begin
        wdata = 0;
    end
end


always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        flag_wr <= 0;
    end
    else if(wr_ack)begin
        flag_wr <= 1;
    end
    else if(end_cnt2)begin
        flag_wr <= 0;
    end
end


always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        cnt2 <= 0; 
    end
    else if(add_cnt2) begin
        if(end_cnt2)
            cnt2 <= 0; 
        else
            cnt2 <= cnt2+1 ;
   end
end
assign add_cnt2 = flag_wr;
assign end_cnt2 = add_cnt2  && cnt2 == 512-1 ;

assign addr = 13'b0;


endmodule

