/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/
module sdram_top_scan(
    clk      ,
    rst_n    ,
    key_col  ,
    key_row  ,
    dq       ,
    cke      ,
    cs       ,  
    ras      ,
    cas      ,
    we       ,
    dqm      ,
    sd_addr  ,
    sd_bank  ,
    sd_clk     
    );

    input              clk     ;
    input              rst_n   ;
    input  [3:0]       key_col ;
    inout  [15:0]      dq      ;
    output             cke     ;    
    output             cs      ;
    output             ras     ;
    output             cas     ;
    output             we      ;
    output [1 :0]      dqm     ;
    output [12:0]      sd_addr ;
    output [1 :0]      sd_bank ;
    output             sd_clk  ;
    output [3 :0]      key_row ;



    wire               cke     ;
    wire               cs      ;
    wire               ras     ;
    wire               cas     ;
    wire               we      ;
    wire  [1 :0]       dqm     ;
    wire  [12:0]       sd_addr ;
    wire  [1 :0]       sd_bank ;
    wire               sd_clk  ;
    wire  [3 :0]       key_row ;



    wire              clk_100m ;
    wire              wr_ack   ;
    wire              rd_ack   ;
    wire              wr_req   ;
    wire              rd_req   ;
    wire [1 :0]       bank     ;
    wire [12:0]       addr     ;
    wire [15:0]       wdata    ;
    wire [15:0]       rdata    ;
    wire              rdata_vld;
    wire [3:0 ]       key_vld  ;

    wire [15:0]       dq_in    ;
    wire [15:0]       dq_out   ;
    wire              dq_out_en;

    assign  dq_in = dq;
    assign  dq    = dq_out_en?dq_out:16'hzzzz;
        


    pll_100m uut_pll(
    	.inclk0 (clk     ),
    	.c0     (clk_100m)
    );

    key_scan uut_key(
        .clk       (clk_100m    ), 
        .rst_n     (rst_n  ), 
        .key_col   (key_col), 
        .key_row   (key_row),
        .key_en    (key_en ),
    );
        
    data_ctrl uut_ctrl(
        .clk       (clk_100m ), 
        .rst_n     (rst_n    ),
        .key_vld   (key_en   ),
        .wr_ack    (wr_ack   ),   
        .rd_ack    (rd_ack   ),   
        .wr_req    (wr_req   ),   
        .rd_req    (rd_req   ),   
        .bank      (bank     ),   
        .addr      (addr     ),    
        .wdata     (wdata    )    
        );
    
    sdram_intf uut_sdram(
        .clk       (clk_100m ),   
        .rst_n     (rst_n    ),
        .wr_req    (wr_req   ),
        .rd_req    (rd_req   ),
        .bank      (bank     ),
        .addr      (addr     ),
        .wdata     (wdata    ),
        .dq_in     (dq_in    ),
        .dq_out    (dq_out   ),
        .dq_out_en (dq_out_en),
        .wr_ack    (wr_ack   ),
        .rd_ack    (rd_ack   ),
        .rdata     (rdata    ),
        .rdata_vld (rdata_vld),
        .cke       (cke      ),
        .cs        (cs       ),
        .ras       (ras      ),
        .cas       (cas      ),
        .we        (we       ),
        .dqm       (dqm      ),
        .sd_addr   (sd_addr  ),
        .sd_bank   (sd_bank  ),
        .sd_clk    (sd_clk   )  
        
    );



    endmodule

